@@ -28,17 +28,30 @@ import egauge.webapi.cloud.credentials
     self.sn_api = webapi.cloud.SerialNumber(auth=auth)
 
 """
+# pylint: disable=too-few-public-methods
+
 import getpass
 
-from PySide2.QtWidgets import QDialog
-from .gui.credentials_dialog import Ui_Credentials_Dialog
+try:
+    from PySide2.QtWidgets import QDialog
+    from .gui.credentials_dialog import Ui_Credentials_Dialog
+
+    have_pyside2 = True
+except ModuleNotFoundError:
+    have_pyside2 = False
+
+    class QDialog:
+        """Dummy class for use by Credentials_Dialog."""
+
+    class Ui_Credentials_Dialog:
+        """Dummy class for use by Credentials_Dialog."""
 
 
 class LoginCanceled(Exception):
     """Raised when the user cancels a login request."""
 
 
-class Credentials_Manager:
+class CredentialsManager:
     def __init__(self, gui_parent=None):
         """Create a credentials manager.  The task of this manager is mainly
         to track if a previous login failed.  The user of this object
@@ -50,7 +63,7 @@ class Credentials_Manager:
         will be requested via standard I/O (getpass).
 
         """
-        self.parent = gui_parent
+        self.parent = gui_parent if have_pyside2 else None
         self.previous_login_failed = False
 
     def ask(self):
@@ -61,9 +74,7 @@ class Credentials_Manager:
 
         """
         if self.parent:
-            dialog = Credentials_Dialog(
-                self.parent, self.previous_login_failed
-            )
+            dialog = CredentialsDialog(self.parent, self.previous_login_failed)
             dialog.exec_()
             if not dialog.accepted:
                 raise LoginCanceled()
@@ -73,7 +84,7 @@ class Credentials_Manager:
         fail_msg = ""
         if self.previous_login_failed:
             fail_msg = "Login failed.  "
-        print("%sPlease enter eGuard credentials." % fail_msg)
+        print(fail_msg + "Please enter eGuard credentials.")
         try:
             usr = input("Username: ")
             pwd = getpass.getpass(prompt="Password[+token]: ")
@@ -83,7 +94,7 @@ class Credentials_Manager:
         return [usr, pwd]
 
 
-class Credentials_Dialog(QDialog, Ui_Credentials_Dialog):
+class CredentialsDialog(QDialog, Ui_Credentials_Dialog):
     def __init__(self, parent, failed):
         self.accepted = False
         self.username = None
@@ -106,3 +117,9 @@ class Credentials_Dialog(QDialog, Ui_Credentials_Dialog):
         self.username = self.username_lineEdit.text()
         self.password = self.password_lineEdit.text()
         self.token = self.token_lineEdit.text()
+
+
+# Aliases for backwards-compatibility.  Please use CredentialsManager
+# and CredentialsDialog in new code.
+Credentials_Manager = CredentialsManager
+Credentials_Dialog = CredentialsDialog
