@@ -27,8 +27,12 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 #
+from .capture import *
 from .ctid_info import *
 from .device import *
 from .local import *
+from .physical_quantity import *
+from .physical_units import *
 from .register import *
 from .register_row import *
+from .virtual_register import *
