@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020 eGauge Systems LLC
+# Copyright (c) 2020, 2022 eGauge Systems LLC
 #       1644 Conestoga St, Suite 2
 #       Boulder, CO 80301
 #       voice: 720-545-9767
@@ -29,11 +29,13 @@
 #
 """This module provides access to the eGauge WebAPI's /api/register
 service."""
-
 import decimal
 import json
 
+from deprecated import deprecated
+
 from ..error import Error
+from .physical_quantity import PhysicalQuantity
 from .register_row import RegisterRow
 
 
@@ -43,10 +45,15 @@ class RegisterError(Error):
 
 def regnames_to_ranges(dev, regs):
     """Convert a list of register names to a register-ranges string."""
-    indices = []
+    indices = set()
     for name in regs:
-        indices.append(dev.reg_idx(name))
-    indices = sorted(indices)
+        formula = dev.reg_formula(name)
+        if formula is None:
+            indices.add(dev.reg_idx(name))
+        else:
+            for phys_reg in formula.phys_regs:
+                indices.add(dev.reg_idx(phys_reg))
+    indices = sorted(list(indices))
 
     ranges = []
     idx = 0
@@ -57,9 +64,9 @@ def regnames_to_ranges(dev, regs):
             stop = indices[idx]
             idx += 1
         if start == stop:
-            ranges.append("%s" % start)
+            ranges.append(str(start))
         else:
-            ranges.append("%s:%s" % (start, stop))
+            ranges.append(str(start) + ":" + str(stop))
     if len(ranges) < 1:
         return None
     return "+".join(ranges)
@@ -67,21 +74,21 @@ def regnames_to_ranges(dev, regs):
 
 class Register:
     def __init__(self, dev, params=None, regs=None, **kwargs):
-        """Fetch register data from device DEV.  PARAMS is a dictionary
-        of query parameters that specify the data to return.  If REGS
-        is none, all registers are returned.  If not None, it must be
-        a list of register names for which data should be returned.
+        """Fetch register data from device DEV.  PARAMS is a dictionary of
+        query parameters that specify the data to return.  If REGS is
+        None, all registers are returned.  If not None, it must be a
+        list of register names for which data should be returned.
         Additional keyword arguments are passed along the
-        requests.get().
-
-        """
+        requests.get()."""
         self.dev = dev
+        # maps regname to index in "register"/"row" arrays:
         self.regorder = None
         self.iter_range_idx = None
         self.iter_row_idx = None
         self.iter_ts = None
         if params is None:
             params = {}
+        self._requested_regs = regs
         if regs is not None:
             reg_ranges = regnames_to_ranges(dev, regs)
             if reg_ranges is not None:
@@ -95,38 +102,61 @@ class Register:
             )
 
     def _create_regorder(self):
+        """Create a dictionary mapping a register name to the index within
+        the `registers' and `row' arrays which contain the info for
+        that particular name."""
         self.regorder = {}
         for index, reg in enumerate(self.raw["registers"]):
             self.regorder[reg["name"]] = index
 
     def ts(self):
-        """Return the timestamp of the register data as a Decimal()."""
+        """Return the timestamp of the register rates as a Decimal()."""
         if self.raw is None:
             return None
         return decimal.Decimal(self.raw["ts"])
 
-    def have(self, reg):
-        """Return True if register REG exists, False otherwise."""
+    @property
+    def regs(self):
+        """The list of available register names."""
+        if self._requested_regs:
+            return self._requested_regs
+
         if self.regorder is None:
             self._create_regorder()
-        return reg in self.regorder
+        return list(self.regorder.keys()) + self.dev.reg_virtuals()
 
-    def rate(self, reg):
-        """Return the rate of change value of the register REG."""
+    def have(self, reg):
+        """Return True if register REG is available, False otherwise."""
+        return reg in self.regs
+
+    def _rate(self, reg) -> float:
         if self.raw is None:
             return None
         if self.regorder is None:
             self._create_regorder()
-        return self.raw["registers"][self.regorder[reg]]["rate"]
+
+        row = self.raw["registers"]
+        formula = self.dev.reg_formula(reg)
+        if formula is not None:
+            return formula.calc(lambda reg: row[self.regorder[reg]]["rate"])
+        return row[self.regorder[reg]]["rate"]
+
+    @deprecated(version="0.7.0", reason="use pq_rate() instead")
+    def rate(self, reg) -> float:
+        """Return the rate of change value of the register REG."""
+        return self._rate(reg)
+
+    def pq_rate(self, reg=None) -> PhysicalQuantity:
+        """Return the rate of change value of the register REG."""
+        rate = self._rate(reg)
+        return PhysicalQuantity(rate, self.type_code(reg), is_cumul=False) \
+            .to_preferred()
 
     def type_code(self, reg):
         """Return the type-code of register REG."""
-        if self.raw is None:
-            return None
-        if self.regorder is None:
-            self._create_regorder()
-        return self.raw["registers"][self.regorder[reg]]["type"]
+        return self.dev.reg_type(reg)
 
+    @deprecated(version="0.7.0", reason="use Device.reg_formula() instead")
     def formula(self, reg):
         """Return the formula of virtual register REG."""
         if self.raw is None:
@@ -152,10 +182,28 @@ class Register:
         return self.raw["registers"][self.regorder[reg]]["did"]
 
     def _make_row(self, ts, cumul_list):
+        """Convert a "rows" array to a RegisterRow()."""
         row = RegisterRow(ts)
+
+        # first, fill in the physical registers:
+        phys = {}
         for idx, cumul in enumerate(cumul_list):
-            name = self.raw["registers"][idx]["name"]
-            row.regs[name] = int(cumul)
+            ri = self.raw["registers"][idx]
+            reg = ri["name"]
+            cumul = int(cumul)
+            phys[reg] = cumul
+            if self._requested_regs and reg not in self._requested_regs:
+                continue
+            row.regs[reg] = cumul
+            row.type_codes[reg] = self.type_code(reg)
+
+        # second, fill in virtual registers (if any):
+        for reg in self.dev.reg_virtuals():
+            if self._requested_regs and reg not in self._requested_regs:
+                continue
+            formula = self.dev.reg_formula(reg)
+            row.regs[reg] = formula.calc(lambda reg: phys[reg])
+            row.type_codes[reg] = self.type_code(reg)
         return row
 
     def __getitem__(self, index):
